import logging
import sys
import argparse
from tempfile import NamedTemporaryFile

from deploy._staging import stage
from deploy._send import send
from deploy._logging_formatter import CustomFormatter

logger = logging.getLogger()


def deploy(ssh_host: str,
           target_dir: str,
           archive_path: str,
           require_confirmation: bool = True):
    """ Runs the deployment pipeline from start to finish.

    Args:
        ssh_host: Exact string that will be used by ssh and rsync as the host.
        target_dir: Exact, absolute directory where files will be placed;
                    anything already present will be clobbered.
        archive_path: Path to the local archive to deploy from
    Returns:
        Nothing.
    """
    logger.setLevel(logging.INFO)
    ch = logging.StreamHandler(sys.stdout)
    ch.setFormatter(CustomFormatter())
    ch.setLevel(logging.INFO)
    logger.addHandler(ch)

    temp_archive: NamedTemporaryFile = stage(archive_path)
    confirm_archive(temp_archive, require_confirmation)
    send(ssh_host, target_dir, temp_archive)


def confirm_archive(temp_archive: NamedTemporaryFile,
                    require_confirmation: bool):
    if require_confirmation:
        logging.warn(f"Temp archive currently exists at: {temp_archive.name}")
        confirm = input("Please confirm archive validity. (y/n)")
        if confirm.upper() != "Y":
            logging.fatal("CONFIRMATION FAILED, ABORT")
            temp_archive.close()
            sys.exit(1)
    else:
        logging.warn("Confirmation skipped.")


def add_deploy_command(top_level_parsers: argparse._SubParsersAction):
    parser: argparse.ArgumentParser = top_level_parsers.add_parser("deploy")
    parser.add_argument("-a", "--archive",
                        help="path to archive",
                        dest="archive_path",
                        nargs=1,
                        required=True)
    parser.add_argument(dest="ssh_host")
    parser.add_argument(dest="target_dir")
    parser.set_defaults(func=lambda a: deploy(ssh_host=a.ssh_host,
                                              target_dir=a.target_dir,
                                              archive_path=a.archive_path[0]))
